% This is the 1st-level estimation for RA
% For any given subject, find the best preference functional
% Given an elicitation method, we rank the preference functionals (by loglikelihood)
% C and E are intermediate matrix

tic
addpath('data from experiment')
addpath('estimation results');
% load data from experiment/trialPC;
data=csvread('data from experiment/m_logliks-All.csv');

ns=96;            % number of subjects.
fs=1;			  % first subject to be estimated.
ls=ns;            % last subject to be estimated.
npt=4;            % this is the number of problem types (1 is PC, 2 is Allocation, 3 is BDM, 4 is Price List)
fpt=1;            % first problem type (PC etc)
lpt=4;            % last problem type

A=data; % This is the original matrix containing all the loglikelihoods
B=0*A; % B is the matrix to save sorted loglikelihoods
D=0*A; % D is the matrix to save sorted preference functionals related of loglikelihoods,
       % 1=RREU; 2=RRRD; 3=AREU; 4=ARRD
       
       
% Sort of Loglikelihodds and Preference Functionals (max to min)
for sn=fs:1:ls % Loop from first subject for estimation to last subject for estimation
    
    B(sn,1)=sn; % Matrix B is for saving sorted loglikelihoods, first col of B is subject number
    D(sn,1)=sn;
    for ptn=fpt:1:lpt % Loop from first problem type for estimation to last problem type for estimation
        clear C E
        C=A(sn,4*ptn-2:4*ptn+1); % This is an intermediate matrix to sort loglikelihood for each problem type
        E=[1 2 3 4]; %dummy of preference functional 1=RREU; 2=RRRD; 3=AREU; 4=ARRD
        for n=1:1:4 %sort the four loglikelihoods max to min
            clear rows cols
            B(sn,4*ptn-2+(n-1))=max(max(C)); %Find the max loglikelihood in current C
            [rows, cols]=find(C==max(max(C))); % Find the row and column of the max loglikelihood
            D(sn,4*ptn-2+(n-1))=E(:,cols(1,1)); % Save the dummy of prefernce functional
            C(:,cols(1,1))=-99999; % cols(1,1) for the case that more than one loglikelihoods to be the max   
        end % of sort the four loglikelihoods max to min
    end % of Loop ptn
end % of Loop sn


%Drop NotConverged/HittingBounds subjects (27 in total)
k=1;
i=0;
while i<ns 
    if B(k,1)==3 || B(k,1)==10 || B(k,1)==20 || B(k,1)==22 || B(k,1)==26 || B(k,1)==29 || B(k,1)==35 || B(k,1)==36....
            || B(k,1)==37 || B(k,1)==45 || B(k,1)==49 || B(k,1)==52 || B(k,1)==53 || B(k,1)==54 || B(k,1)==55....
            || B(k,1)==60 || B(k,1)==62 || B(k,1)==65 || B(k,1)==67 || B(k,1)==81 || B(k,1)==83 || B(k,1)==85....
            || B(k,1)==89 || B(k,1)==91 || B(k,1)==92 || B(k,1)==94 || B(k,1)==96
        B(k,:)=[];
        D(k,:)=[];
        k=k+0;
    else
        k=k+1;
    end
    i=i+1;
end 

%For each subject, we find the best preference functionals.
F=zeros(size(D,1),10); %F is an matrix to record the best fitting preference functional for each specific subject
C=zeros(1,4); % C is an intermediate matrix to save the best (with biggest loglikelihood) functional dummy of the four elicitation methods
for i=1:1:size(D,1)
    clear C E
    for k=1:1:4
       C(1,k)=D(i, 4*k-2);
    end
    E=tabulate(C(:));
    [rows,cols]=find(E==max(max(E(:,3))));
    F(i,1)=D(i,1);
    k=1;
    while k<=length(rows)
        F(i,k+1)=E(rows(k,1),1);
        F(i,k+5)=max(max(E(:,3)))/100;
        k=k+1;
    end
end
%Show estimation results in text
clear C E
C=tabulate(F(:,2));
E=tabulate(F(:,3));
disp(['RRRD is the best fitted preference functional for ', num2str(C(2,2)), ' subjects']);
disp(['ARRD is the best fitted preference functional for ', num2str(C(4,2)+E(2,2)), ' subjects']);
disp(['RRRD and ARRD are both the best fitted preference functionals for ', num2str(E(2,2)), ' subjects']);
%show estimation results in figure
PF=[C(2,2)-E(2,2) C(4,2) E(2,2)]; % Best fitted preference functionals, RRRD, ARRD, BOTH
ph=pie3(PF); 
legend(ph(2:4:end),{'RRRD','ARRD','RRRD&ARRD'});
title('Proportion of Best Fitted Preference Functionals');


resultspath=[pwd,'\estimation results\'];
csvwrite([resultspath,'m_logliks-sorted.csv'],B);
csvwrite([resultspath,'m_functionals-sorted.csv'],D);
csvwrite([resultspath,'m_functionals-bestfitted.csv'],F);
toc