% This is a simulation for Allocation

tic
rp=1; %repeat time for simulation
r=0.5;
s=1.5;
g=1.5;
addpath('data from experiment')
addpath('estimation results');
% load data from experiment/trialPC;
data=dlmread('data from experiment/AllocationParameters.txt',''); 

np=81;           % number of allocation questions.
nft=4;            % this is the number of preference functionals (1 is RREU, 2 is RRRD, 3 is AREU, 4 is ARRD)
fft=1;            % first preference functional type (RREU etc)
lft=3;            % last preference functional type

m=100; % The endowment
p=data(:,3); % probability of Red
e1=data(:,5); % exchange rate on Red
e2=data(:,6); % exchange rate on Yellow

lb(1)=-1.0;     ub(1)=5.0;  st(1)=0.5;                 % r in RR starts with risk-averse
lb(2)=-1.0;     ub(2)=2.0;  st(2)=0.01;                % r in AR starts with risk-averse
lb(3)=0.6;      ub(3)=1.7;  st(3)=1.0;                 % g in RD starts at EU
lb(4)=0.01;     ub(4)=2.0;  st(4)=0.2;                 % s in normal

xoRREU=ones(np,1); % The optimal allocation on Red for RREU
xoRRRD=ones(np,1); % The optimal allocation on Red for RRRD
xoAREU=ones(np,1); % The optimal allocation on Red for AREU
xoARRD=ones(np,1); % The optimal allocation on Red for ARRD
xaRREU=ones(np,1); % The simulated actual allocation on Red for RREU
xaRRRD=ones(np,1); % The simulated actual allocation on Red for RRRD
xaAREU=ones(np,1); % The simulated actual allocation on Red for AREU
xaARRD=ones(np,1); % The simulated actual allocation on Red for ARRD

 options = psoptimset('MaxIter',100000000);
 
A=ones(rp,1); % r
B=ones(rp,1); % g
C=ones(rp,1); % s
D=ones(rp,1); % log likelihoods
for k=1:1:rp
for pfn=fft:1:lft
    if pfn==1  % RREU for AL
            for i=1:1:np % Simulate the allocation amount
                xoRREU(i)=m*p(i)^(1/r)*e1(i)^((1-r)/r)/(p(i)^(1/r)*e1(i)^((1-r)/r)+(1-p(i))^(1/r)*e2(i)^((1-r)/r));
                xaRREU(i)=xoRREU(i)+normrnd(0,1/s);
                if xaRREU(i)<0
                    xaRREU(i)=0;
                end
            end % End Simulate the allocation amount
            
            % Estimate the parameters of the simulation
            xa=xaRREU;
            lbs=[lb(1),lb(4)];    %Lower bounds on r and s
            ubs=[ub(1),ub(4)];    %Upper bounds on r and s 
            pars0=[st(1),st(4)];  %Start values of r and s
            [xval,fval,exitflag]=patternsearch(@(z) ll21(z,xa,p,e1,e2,m),pars0,[],[],[],[],lbs,ubs,[],options);
            A(k)=xval(1);
            C(k)=xval(2);
            D(k)=-fval;
            % End Estimate the parameters of the simulation
    end % End RREU for AL
    
    if pfn==2 %RRRD for AL
        for i=1:1:np % Simulate the allocation amount
            clear q1 q2 xo1 xo2 xo3
            q1=wf(p(i),g);
            q2=wf((1-p(i)),g);
            xo1=m*q1^(1/r)*e1(i)^((1-r)/r)/(q1^(1/r)*e1(i)^((1-r)/r)+(1-q1)^(1/r)*e2(i)^((1-r)/r));
            xo2=m*(1-q2)^(1/r)*e1(i)^((1-r)/r)/((1-q2)^(1/r)*e1(i)^((1-r)/r)+q2^(1/r)*e2(i)^((1-r)/r));
            xo3=m*p(i)^(1/r)*e1(i)^((1-r)/r)/(p(i)^(1/r)*e1(i)^((1-r)/r)+(1-p(i))^(1/r)*e2(i)^((1-r)/r));
            
            if xo1*e1(i)<(1-xo1)*e2(i)
                xoRRRD(i)=xo1;
            elseif xo2*e1(i)>1-(xo2)*e2(i)
                xoRRRD(i)=xo2;
            elseif xo3*e1(i)==(1-xo3)*e2(i)
                xoRRRD(i)=xo3;
            else
                disp(xo1);
                disp(e1(i));
                disp(e2(i));
                disp(xo1*e1(i));
                disp((1-xo1)*e2(i));
                disp('xo Error')
                break
            end
            xaRRRD(i)=xoRRRD(i)+normrnd(0,1/s);
             if xaRRRD(i)<0
                    xaRRRD(i)=0;
             end
        end % End Simulate the allocation amount
            
            % Estimate the parameters of the simulation
            xa=xaRRRD;
            lbs=[lb(1),lb(3),lb(4)];    %Lower bounds on r, g and s
            ubs=[ub(1),ub(3),ub(4)];    %Upper bounds on r, g and s
            pars0=[st(1),st(3),st(4)];  %Start values of r, g and s
            [xval,fval,exitflag]=patternsearch(@(z) ll22(z,xa,p,e1,e2,m),pars0,[],[],[],[],lbs,ubs,[],options);
            A(k)=xval(1);
            B(k)=xval(2);
            C(k)=xval(3);
            D(k)=-fval;
            % End Estimate the parameters of the simulation
    end % End RRRD for AL
    
    if pfn==3 %AREU for AL
        for i=1:1:np % Simulate the allocation amount
                xoAREU(i)=(m*e2(i)+(log(p(i)*e1(i)/((1-p(i))*e2(i)))/r))/(e1(i)+e2(i));
                xaAREU(i)=xoAREU(i)+normrnd(0,1/s);
                if xaAREU(i)<0
                    xaAREU(i)=0;
                end
        end % End Simulate the allocation amount
        
        % Estimate the parameters of the simulation
        xa=xaAREU;
        lbs=[lb(2),lb(4)];     %Lower bounds on r and s
        ubs=[ub(2),ub(4)];     %Upper bounds on r and s
        pars0=[st(2),st(4)];   %Start values of r and s
        [xval,fval,exitflag]=patternsearch(@(z) ll23(z,xa,p,e1,e2,m),pars0,[],[],[],[],lbs,ubs,[],options);
        A(k)=xval(1);
        C(k)=xval(2);
        D(k)=-fval;
        % End Estimate the parameters of the simulation
        
    end % %AREU for AL
    
    if pfn==4 %ARRD for AL
        for i=1:1:np % Simulate the allocation amount
            clear q1 q2 xo1 xo2 xo3
            q1=wf(p(i),g);
            q2=wf((1-p(i)),g);
            xo1=(m*e2(i)+(log(q1*e1(i)/((1-q1)*e2(i)))/r))/(e1(i)+e2(i));
            xo2=(m*e2(i)+(log((1-q2)*e1(i)/(q2*e2(i)))/r))/(e1(i)+e2(i));
            xo3=(m*e2(i)+(log(p(i)*e1(i)/((1-p(i))*e2(i)))/r))/(e1(i)+e2(i));
            
            if xo1*e1(i)<(1-xo1)*e2(i)
                xoARRD(i)=xo1;
            elseif xo2*e1(i)>1-(xo2)*e2(i)
                xoARRD(i)=xo2;
            elseif xo3*e1(i)==(1-xo3)*e2(i)
                xoARRD(i)=xo3;
            else
                disp('xo Error')
                break
            end
            xaARRD(i)=xoARRD(i)+normrnd(0,1/s);
        end % End Simulate the allocation amount
        
        % Estimate the parameters of the simulation
            xa=xaARRD;
            lbs=[lb(2),lb(3),lb(4)];     %Lower bounds on r, g and s
            ubs=[ub(2),ub(3),ub(4)];     %Upper bounds on r, g and s
            pars0=[st(2),st(3),st(4)];   %Start values of r, g and s
            [xval,fval,exitflag]=patternsearch(@(z) ll24(z,xa,p,e1,e2,m),pars0,[],[],[],[],lbs,ubs,[],options);
            A(k)=xval(1);
            B(k)=xval(2);
            C(k)=xval(3);
            D(k)=-fval;
        % End Estimate the parameters of the simulation
    end % %ARRD for AL
end
end
            disp('r');
            disp(mean(A,1));
            disp('g');
            disp(mean(B,1));
            disp('s');
            disp(mean(C,1));
            disp('logliks');
            disp(mean(D,1));

toc